<html>
    <head>
        <meta http-equiv="Content-type" content="text/html; charset=utf-8" />
        <title>$project_name - Index Page</title>
    </head>
    <body>
        <p>It WORKS!!!</p>
        <p>Page generated at {{ date }}.</p>
    </body>
<html>
